// Tree-like
// draws a tree-like shape at the center of the artboard
// ### notice
// The tree is made with various width of lines.
// So please be careful about the setting of Scale Tool
// when you want to scale it.
// 2005.01.12
// 2006.07.19 clean the code, change the variable names to english.
//            Maybe there is still room for a rearrangement to the parameters.
// 2010-05-31 modify to work with cs5
//2012.01.13 (1/13/2012) hacks by CRG:
//                  UI (using Boethos [crgreen.com/boethos])
//                  Cheat for 'persistence' of parameters by naming tree layer group, reading/parsing last (top) layer group.
//                  This is so you can make a ('select menu item') action to run the script, and use it over and over again
//                  with the same settings (as long as the previous tree group is selected)
//                  late 1/13 ... added deselect of previous for efficient make and move, make and move, make and move ...
//                  late 1/14 ... cosmetic changes (last parameter slider), logarithmic slider for last parameter,
//                     much smarter slider code, much smarter in how numbers are handled. Also locks previous tree group
//                     so new one is much easier to move when created. 
// JavaScript Script for Adobe Illustrator CS
// by SATO Hiroyuki  http://park12.wakwak.com/~shp/lc/et/en_aics_script.html
// == Settings =====================================
var grow = 2; // rate for growing
var bend = 2; // rate for bending
var branch = 4; // rate for branching
var thickness = 16; // starting value of the thickness of the trunk(branch)
var minThickness = 0.5; // minimum value of the thickness of the branches
var rate = 0.7; // rate for the thickness of the next branch
var trial = 500; // trail count to grow branches
// ===================CRG hack
if (!documents.length<1) {
    if(activeDocument.activeLayer.locked || !activeDocument.activeLayer.visible){
        alert("Please unlock and show the active layer.");
    } else {
        var win = new Window('dialog', 'Tree-like Options',[260,170,618,528]);
        var w = buildUI();
        if (w != null) {
            w.show();
        }
    }
}
function buildUI() {
    if (win != null) {
        
        //parse
        //propogate into window
        
        grItz = activeDocument.activeLayer.groupItems ;
        
        if (!grItz.length == 0) {
            if (grItz[0].selected) {
                topGroupName = (grItz[0].name);
                if (topGroupName.length > 19) {
                    if (topGroupName.substring(0, 6) == "tree [") {
                        if (topGroupName.substring(topGroupName.length, topGroupName.length-1) == "]") {
                            splitMe = topGroupName.substring(6, topGroupName.length-1);
                            paramArray = splitMe.split("|");
                            if (paramArray.length == 7) {
                                //ok! ready to propagate!
                                //well, i really should check these are all valid numbers ...
                                var badNumFlag = 0;
                                for(var paramIter=0; paramIter<paramArray.length; paramIter++) {
                                    badNumFlag = ( badNumFlag + checkNumber(paramArray[paramIter]) ) ;
                                }
                                //finally...
                                if (badNumFlag == 0) {
                                    grow = parseFloat(paramArray[0]);
                                    bend = parseFloat(paramArray[1]);
                                    branch = parseFloat(paramArray[2]);
                                    thickness = parseFloat(paramArray[3]);
                                    minThickness = parseFloat(paramArray[4]);
                                    rate = parseFloat(paramArray[5]);
                                    trial = parseFloat(paramArray[6]);
                                    //instead of just deselecting, let's lock it to make the new one easily movable
                                    grItz[0].locked =true;
                                }
                            }
                            
                        }
                    }
                }
            }
        }
        
        win.groSlider = win.add('slider', [24,38,154,68], grow, .5, 25);
        //====
        win.groLbl = win.add('statictext', [34,20,164,40], 'Grow: ' + grow.toString());
        win.groLbl.justify = 'left';
        win.bendLbl = win.add('statictext', [34,80,164,100], 'Bend: ' + bend.toString());
        win.bendLbl.justify = 'left';
        win.bendSlider = win.add('slider', [24,98,154,128], bend, .5, 300);
        //====
        win.branchLbl = win.add('statictext', [34,140,164,160], 'Branch: ' + branch.toString());
        win.branchLbl.justify = 'left';
        win.branchSlider = win.add('slider', [24,158,154,188], branch, .5, 50);
        //====
        win.thickSlider = win.add('slider', [204,38,334,68], thickness, 1, 30);
        //====
        win.thickLbl = win.add('statictext', [204,20,334,40], 'Thickness: ' + thickness.toString());
        win.thickLbl.justify = 'left';
        win.minThickLbl = win.add('statictext', [204,80,334,100], 'Minimum Thickness: ' + minThickness.toString());
        win.minThickLbl.justify = 'left';
        win.minThickSlider = win.add('slider', [204,98,334,128], minThickness, .1, 1);
        //====
        win.rateLbl = win.add('statictext', [204,140,334,160], 'Rate: ' + rate.toString());
        win.rateLbl.justify = 'left';
        win.rateSlider = win.add('slider', [204,158,334,188], rate, .1, 1);
        //====
        //don't like the way this slider goes from 1 to 10 ...
        //making it logarithmic
        win.trialLbl = win.add('statictext', [124,210,254,230], 'Complexity: ' + trial.toString());
        win.trialLbl.justify = 'left';
        
        /// sorry for making this line so horribly unreadable. it's the reverse function of logslider (also derived from stackoverflow.com)
        win.trialSlider = win.add('slider', [124-90,228,254+70,258], ((Math.log(trial)-Math.log(1))/((Math.log(1000)-Math.log(1)) / (1000-1)) + 1), 1.0, 1000);
        //====
        
        win.ctl_button5 = win.add('button', [247,307,326,338], 'Make Tree');
        
        win.hintLbl = win.add('statictext', [34,274,334,300], '(De-select top tree group to use default values)');
        win.hintLbl.justify = 'center';
        
        //on make button click
        win.ctl_button5.onClick = function () {
            
            ////get values:
            grow = win.groSlider.value;
            bend = win.bendSlider.value;
            branch = win.branchSlider.value;
            
            thickness = win.thickSlider.value;
            
            mtNum = win.minThickSlider.value;
            minThickness = mtNum;
            
            rNum = win.rateSlider.value;
            rate = rNum;
            
            var trialFromText = win.trialLbl.text;
            
            //ok. i admit it. it's a little kludgey
            trial = parseFloat(trialFromText.substring(12, trialFromText.length));
            
            /////////////////////////amounts to 'save' in group name
            /////////////////////////(allowing precision of 3rd places r of decimal point)
            inish_grow = (Math.round(grow*1000))*.001;
            inish_bend = (Math.round(bend*1000))*.001;
            inish_branch = (Math.round(branch*1000))*.001;
            inish_thickness = (Math.round(thickness*1000))*.001;
            inish_mtNum = (Math.round(minThickness*1000))*.001;
            inish_rNum = (Math.round(rate*1000))*.001;
            inish_trial = (Math.round(trial*1000))*.001;//actually, unnecessary at this point, but doesn't hurt
            /////////////////////////
            ////
            this.parent.close(1);
            g_origin = Origin();
            
            makeTree();
            
            g_origin.restore();
        };
        
        win.groSlider.onChanging = function() {
            var val = (win.groSlider.value);
            win.groLbl.text = ("Grow: " + val.toString() );
        };
        
		win.bendSlider.onChanging = function() {
            var val = (win.bendSlider.value);
            win.bendLbl.text = ("Bend: " + val.toString() );
        };
        
        win.branchSlider.onChanging = function() {
            var val = (win.branchSlider.value);   
            win.branchLbl.text = ("Branch: " + val.toString() );
        };
        
        win.thickSlider.onChanging = function() {
            var val = Math.round(win.thickSlider.value);
            win.thickLbl.text = ("Thickness: " + val.toString() );
        };
        
        win.minThickSlider.onChanging = function() {
            var val = (win.minThickSlider.value);
            win.minThickLbl.text = ("Minimum Thickness: " + val.toString() );
        };
        
        win.rateSlider.onChanging = function() {
            var val = (win.rateSlider.value);
            win.rateLbl.text = ("Rate: " + val.toString() );
        };
        
        win.trialSlider.onChanging = function() {
            var val =  logslider(win.trialSlider.value, 1, 1000);
            win.trialLbl.text = ("Complexity: " + val.toString() );
        };
        
    }
    return win
}
// ===================
//crg hack added funcs
function checkNumber(thisNum) {
    retVal = 0;
    asNum=parseFloat(thisNum);
    if ( isNaN(asNum) ) {retVal = 1;}
    return retVal;
}
//thanks to stackoverflow.com
function logslider(value, min, max) {
  var minv = Math.log(min);
  var maxv = Math.log(max);
  // calculate adjustment factor
  var scale = (maxv-minv) / (max-min);

  var logVal = Math.exp(minv + scale*(value-min));
   return (Math.round(logVal*1000))*.001;
}

// =============================================
// define a growth unit for thickness of a branch
function def_growLen(thickness, minThickness) {
    return Math.max(Math.sqrt(thickness*2), minThickness*3);
}
// define a trial count for thickness of a branch
function def_branchTimes(thickness){ return Math.max(thickness*3, 20); }
// define maximum bending angle
function def_eda_kakudo(growLen){ return Math.PI / growLen / 2; }
var ver10 = version.indexOf('10') == 0;
var ver15_or_later = parseFloat(version.substr(0, 2)) >= 15;
// ---------------------------------------------------------
// A fix for the change of the document origin in CS5,
// and for the multi-artboard in CS4 or later.
// This function locates the active artboard's origin on the
// bottom left at first, and restores it at the end.
//
// USAGE:
// var g_origin = Origin();
// ... (do something in legacy style)
// g_origin.restore();
function Origin(){
    this.ver15_or_later = (parseFloat(version.substr(0, 2)) >= 15); // CS5 or later
    this.ver14 = (version.substr(0, 2) == "14"); // CS4
    
    if(this.ver15_or_later){
        this.saved_coord_system = app.coordinateSystem;
        app.coordinateSystem = CoordinateSystem.ARTBOARDCOORDINATESYSTEM;
        
        var idx  = app.activeDocument.artboards.getActiveArtboardIndex();
        this.ab  = app.activeDocument.artboards[idx];
        
        var o   = this.ab.rulerOrigin;
        var r   = this.ab.artboardRect;
        this.saved_origin = [o[0], o[1]];
        this.ab.rulerOrigin = [0, r[1] - r[3]];
        
    } else if(this.ver14){
        var o = app.activeDocument.rulerOrigin;
        this.saved_origin = [o[0], o[1]];
        app.activeDocument.rulerOrigin = [0, 0];
    }
    
    this.restore = function(){
        if(this.ver15_or_later){
            this.ab.rulerOrigin = this.saved_origin;
            app.coordinateSystem = this.saved_coord_system;
            
        } else if(this.ver14){
            app.activeDocument.rulerOrigin = this.saved_origin;
        }
    };
    
    return this;
}
// ---------------------------------------------------------
//crg -- now moved to make button
//var g_origin = Origin();
//makeTree();
//g_origin.restore();
// =============================================
function makeTree(){
    var lay = activeDocument.activeLayer;
    var branchTimes = def_branchTimes(thickness);
    var growLen = def_growLen(thickness, minThickness);
    
    bend  += grow; branch += bend; grow /= branch; bend /= branch;
    
    // values for the position of the branches
    var branchLocs = [1,-1,0,-0.5,0.5];
    
    // =============================================
    var gr = lay.groupItems.add();
    var branches = [];
    branches.push( [[0,0], [0,growLen], thickness]);
    
    var b, x, y, reserve, c, i, j, r, t, v;
    var newBranches, total, togglekey;
    
    for(var k=0; k<=trial; k++){
        if (branches.length<1) break;
        
        b = branches.shift();
        thickness = b.pop();
        x = b[1][0];
        y = b[1][1];
        reserve = 0;
        c = 1;
        
        // define the growing length anew
        branchTimes = def_branchTimes(thickness);
        growLen = def_growLen(thickness, minThickness);
        
        // grow/bend a branch until it branches
        for(j=0; j <= branchTimes; j++){
            r = Math.random();
            
            if (r < grow){
                reserve += growLen;
                
            } else if (r < bend){
                t = getRad(b[c-1],b[c]) + (Math.random()-0.5) * def_eda_kakudo(growLen);
                v = pointForAngle(t, growLen + reserve);
                // restrain the branch from turning to lower direction
                if (b[c][1] < b[c-1][1]) v[1] *= -0.5;
                x += v[0];
                y += v[1];
                reserve = 0;
                c++;
                b.push([x, y]);
                
            } else { // branch
                if (c < 5) continue; // doesn't branch if it's short yet
                break;
            }
        }
        
        // the following is a branching procedure
        if (thickness <= minThickness) continue;
        
        // define the thickness of branches to grow at random and make an array
        newBranches = [];
        total = 0;
        while(total < thickness){
            // define the thickness of the branch
            r = Math.random() * (thickness-minThickness) * rate + minThickness;
            total += r;
            newBranches.push(r);
        }
        newBranches.sort();
        newBranches.reverse(); // assign a place from thicker value
        
        t = Math.atan2(b[c][0] - b[c-1][0],
            b[c-1][1] - b[c][1]); // angle of the parent branch
        
        for(i=0; i<newBranches.length; i++) {
            togglekey = branchLocs[i%(branchLocs.length)];
            v = pointForAngle(t, (thickness - newBranches[i])/2*togglekey);
            branches.push( [b[c-1],
                    [b[c][0] + v[0], b[c][1] + v[1]],
            newBranches[i]]);
        }
        
        // draw
        if (b.length > 3) {
            with(lay.pathItems.add()){
                setEntirePath(b);
                filled = false;
                stroked = true;
                strokeWidth = thickness;
                strokeCap = StrokeCap.BUTTENDCAP;
                strokeJoin = StrokeJoin.ROUNDENDJOIN;
                if(ver10){
                    moveToBeginning(gr);
                } else {
                    move(gr, ElementPlacement.PLACEATBEGINNING);
                }
            }
        }
    }
    
    // translate group to the center of the artboard
    if(gr.pathItems.length>0){
        gr.translate(activeDocument.width/2, activeDocument.height/2);
        //crg hack:
        ///////////
        // adds parameters to group name, selects group
        // (If you want defaults, de-select top group)
        gr.name = ( "tree [" + inish_grow.toString() + "|" + 
            inish_bend.toString() + "|"+ inish_branch.toString() + "|" + 
            inish_thickness.toString() + "|" + inish_mtNum.toString() + "|" + 
            inish_rNum.toString() + "|" + inish_trial.toString() + "]");
        gr.selected = true;
  } else { gr.remove(); }
}
// ----------------------------------------------
function pointForAngle(t, m){
    return [Math.cos(t)*m, Math.sin(t)*m];
}
// ----------------------------------------------
function getRad(p1, p2) {
    return Math.atan2(p2[1] - p1[1], p2[0] - p1[0]);
}
// ------------------------------------------------
function normalize(p, o){
    var d = dist(p,o);
    return d==0 ? [0,0] : [(p[0]-o[0])/d, (p[1]-o[1])/d];
}
// ----------------------------------------------
// return the distance between p1 and p2
function dist(p1, p2) {
    return Math.sqrt(Math.pow(p1[0] - p2[0],2) + Math.pow(p1[1] - p2[1],2));
}
// ----------------------------------------------
